@@ -5,7 +5,7 @@ use Carp;
 use Tie::File;
 use Fcntl qw(:Fcompat :DEFAULT :flock); 
 use vars qw($VERSION);
-$VERSION = 0.6;
+$VERSION = 0.63;
 
 our $file_passwd = '/etc/passwd';
 our $file_shadow = '/etc/shadow';
@@ -34,9 +34,9 @@ sub fields { keys %field }
 sub new {
 	my $class = shift;
 	my $user = shift;
+	croak "no such user" unless _exists($user);
 	(my @args) = _read_user($user, $file_passwd, 1);
 	@args =	(@args, _read_user($user, $file_shadow, 0));
-	croak "no such user" unless scalar @args > 0;
 	return bless [ @args ], ref($class)||$class;
 }
 
@@ -55,9 +55,11 @@ sub set {
 	return 0 unless exists($field{$what});
 	return 0 unless my $newval = shift;
 	return 0 if $newval =~ /:/ and $field{$what} != 8; 
+	$newval = '' if $newval eq 'undef';
 	my $flag = shift || 0;
 	my $oldval = $self->[$field{$what}];
 	my $name = $self->[$field{NAME}];
+	_clean($name);
 	$self->[$field{$what}] = $newval;
 	if($field{$what} <= 6){
 		my @file = _io_file("$file_passwd", '', 'r');
@@ -80,7 +82,7 @@ sub set {
 	if($field{$what} > 6){	
 		my @file = _io_file("$file_shadow", '', 'r');
 		$self->[9] = _get_1970_diff() if $field{$what} == 8;
-		if($field{$what} == 8){
+		if($field{$what} == 8 && $newval){
 			$self->[8] = _gen_pass($self->[$field{$what}]) unless $flag
 		}
 		my @user;
@@ -108,7 +110,7 @@ sub _read_user {
 	my (@user, @file);
 	@file = _io_file($file, '', 'r');
 	for(@file){
-		/^$username/ or next;
+		/^(.[^:]*):/ && $1 eq $username or next;
 		my $user = $_;
 		if($flag){
 			for(1..7){
@@ -118,7 +120,7 @@ sub _read_user {
 				$user[$_ - 1] = $ss;
 			}
 		}else{
-			for(1..8){
+			for(1..9){
 				$user =~ m#(.[^:]*){$_}#;
 				my $ss = $1;
 				$ss =~ s/(^:*|:*$)//;
@@ -142,8 +144,9 @@ sub _gen_pass {
 sub _exists {
 	my $username = shift || die "no usrename given";
 	my @file = _io_file("$file_passwd", '', 'r');
-	/^$username/ and (my @fields) = split /:/, $_ for @file;
-	return scalar @fields;
+	my $flag;
+	/^(.[^:]*):/ and $1 eq $username and $flag = 1 for @file;
+	return $flag ? 1 : 0
 }
 
 sub add {
@@ -221,13 +224,35 @@ sub del{
 	_exists($username) or croak "$username does not exist";
 	my @old = _io_file("$file_passwd", '', 'r');
 	my @new;
-	/^$username:/ or push @new, $_ for @old;
+	/^(.[^:]*):/ and $1 eq $username or push @new, $_ for @old;
 	_io_file("$file_passwd", \@new, 'w');
 	@new = ();
 	@old = _io_file("$file_shadow", '', 'r');
-	/^$username:/ or push @new, $_ for @old;
+	/^(.[^:]*):/ and $1 eq $username or push @new, $_ for @old;
 	_io_file("$file_shadow", \@new, 'w');
-	return 1
+}
+
+sub tobsd{
+	my $self = shift;
+	(my @file) = _io_file("$file_shadow", '', 'r');
+	my $name = $self->get('name');
+	my @user;
+	for(@file){
+		/^$name:/ or next;
+		push @user, $name, ':';
+		push @user, $self->get('password'), ':';
+		push @user, $self->get('uid'), ':';
+		push @user, $self->get('gid'), ':';
+		push @user, ':';
+		push @user, $self->get('expire') || 0, ':';
+		push @user, $self->get('expire') || 0, ':';
+		push @user, $self->get('comment'), ':';
+		push @user, $self->get('home'), ':';
+		push @user, $self->get('shell');
+		my $user = join '', @user;
+		s/.*/$user/;
+	}
+	_io_file("$file_shadow", \@file, 'w');
 }
 
 sub _io_file{
@@ -258,11 +283,20 @@ sub _io_file{
 	}
 }
 	
+sub users{
+	my $class = shift;
+	(my @file) = _io_file("$file_passwd", '', 'r');
+	my (%users, @users);
+	m#^(.[^:]+):# and push @users, $1 for @file;
+	map{ $users{$_} = 1 }@users;
+	return %users
+}
+	
 sub lock{
 	my $self = shift;
 	my $password = $self->get("password");
 	return 1 if $password =~ /^\!/;
-	$password =~ s/(.+)/!$1/;
+	$password =~ s/(.*)/!$1/;
 	$self->set("password", $password, 1);
 }
 
@@ -270,12 +304,19 @@ sub unlock{
         my $self = shift;
 	my $password = $self->get("password");
 	return 1 if $password !~ /^\!/;
-	$password =~ s/^\!(.+)/$1/;
+	$password =~ s/^\!//;
+	$password or $password = 'undef';
         $self->set("password", $password, 1);
 }
 
 sub _get_1970_diff{ return int time / (3600 * 24) }
 
+sub _clean{
+	my $specchars = \shift;
+	my $special = qr#\$|\*|\@|\^|\+|\.|\?|\)|\(|\||\]|\[|\{|\}#;
+	$$specchars =~ s/($special)/\\$1/g;
+}
+
 1
 
 __END__
@@ -325,19 +366,30 @@ are not in "/etc" directory.
 
 =over 4
 
-=item B<new> (username)
+=item B<new> 
+	(username)
+
+
+=item B<add> 
+	(username, ...)
 
-=item B<add> (username, ...)
 
 Class method - add new user account
 arguments to add are optional, except username;
 they may be (username, password, uid, gid, comment, home, shell)
 
-=item B<del> (username)
+=item B<del> 
+	(username)
+
 
 Class method - removes user account
 
-=item B<get> get one of the following fields:
+=item B<tobsd> 
+	converts user fields in shadow / master.passwd file to bsd style
+
+
+=item B<get> 
+	get one of the following fields:
 	
 =over 4
 
@@ -393,18 +445,28 @@ or 15 - Currently not used
 
 either string or number can be argument
 
-=item B<set> (field) 
+
+=item B<set> 
+	(field) 
 
 set a field which must be string, but not a number
 
-=item B<lock> (username)
+
+=item B<lock> 
+	(username)
 
 Lock user account (puts '!' at the beginning of the encoded password)
 
-=item B<unlock>
 
+=item B<unlock>
+	(username)
+	
 Unlock user account (removes '!' from the beginning of the encoded password)
 
+=item B<users>
+
+Class method - return hash which keys are all users, teken from $file_passwd
+
 =head1 FILES
 
 B</etc/passwd>, B</etc/shadow> unless given your own B<passwd> and B<shadow> files
